.input[data-v-b99ddb7a]{border:1px solid var(--border-color);border-radius:5px;overflow:hidden}.info[data-v-b99ddb7a]{background-color:var(--grey);padding:5px 10px}.value[data-v-b99ddb7a]{border-left:1px solid var(--border-color);padding:5px 10px}.folder[data-v-b99ddb7a]{padding:5px;background-color:var(--grey);border-left:1px solid var(--border-color);cursor:pointer}input[data-v-b99ddb7a]{border:none}.create-option[data-v-3db3d89b]{text-align:left;padding-bottom:10px;padding-top:10px;border-bottom:1px solid var(--border-color)}.create-btn[data-v-3db3d89b]{text-align:center;background-color:#f0f0f0;height:36px;padding:6px;border-radius:8px;margin-top:6px;width:100px;cursor:pointer;color:#2d2d2d}.create-btn[data-v-3db3d89b]:hover{background-color:#e3e3ff;color:#000}.highlight[data-v-3db3d89b]{background-color:#aaf;color:#fff}.create-title[data-v-3db3d89b]{font-size:20px}.create-explanation[data-v-3db3d89b]{font-size:15px}.dimmed-2[data-v-784dc1f1]{color:#5a5a5a}.nowrap[data-v-784dc1f1]{white-space:nowrap}.window[data-v-784dc1f1]{width:100vw;height:100vh}.project-menu[data-v-784dc1f1]{height:100%;width:350px;padding:25px 0 25px 25px;background-color:#f6f6f7;color:#2d2d2d;border-right:1px solid var(--border-color);overflow-y:scroll}.project[data-v-784dc1f1]{padding:10px;cursor:pointer}.project[data-v-784dc1f1]:hover{background-color:#e8e8ff;border-radius:10px}.main-menu[data-v-784dc1f1]{height:100%;background-color:#fff;text-align:center;padding:15px}.project-option[data-v-784dc1f1]{width:20px;margin:0 15px;text-align:center;cursor:pointer}.icon[data-v-784dc1f1]{font-size:100px;line-height:100px;margin-top:50px}.create-menu[data-v-784dc1f1]{width:500px;margin:auto}.plugin-preview[data-v-784dc1f1]{text-align:left;font-size:13px;color:#575757}.add-btn[data-v-784dc1f1]{padding:4px;font-size:15px;color:#323232}
